/*
Copyright 2007 Jonathan Irons

This file is part of JuiceFlag.

JuiceFlag is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JuiceFlag is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JuiceFlag; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

import java.io.*;
import java.util.ArrayList;
import java.lang.Character;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

class MapFile {

//	private MapHeader header;
//	private MapTrailer trailer;
//	private ArrayList<MapLevel> levels;
        private short levels;
        private int[] flags;
        private String[] levelNames;
        private boolean[] flagValues;
        private RandomAccessFile readFile;
        private long mapLength;


        public MapFile(String inFileString) {

                try {
                File inFile = new File(inFileString);
                RandomAccessFile readFile = new RandomAccessFile(inFile, "r");
                readFile.seek(72);
                int mapLength = readFile.readInt();
                short numLevels = readFile.readShort();
                this.levels = numLevels;
                this.flags = new int[this.levels];
                this.levelNames = new String[this.levels];
                long fileLength = readFile.length();
                long difference = fileLength - mapLength;
                this.mapLength = mapLength;

                readFile.seek(fileLength);

                for(int i=0; i < numLevels; i++) {
                        //read trailer for each level:
                        //set seek position to level num * 84 (length of each trailer)
                        long seekPos = mapLength + (i * 84);
                        readFile.seek(seekPos);

                        int levelOffset = readFile.readInt();
                        int levelLength = readFile.readInt();
                        int skips = readFile.skipBytes(8);
                        int flags = readFile.readShort();
                        this.flags[i] = flags;
                        String levelOverheadName = "";
                        byte present = readFile.readByte();
                        while (present != 0) {
                                char newChar = (char) present;
                                levelOverheadName = levelOverheadName + newChar;
                                present = readFile.readByte();
                        }
                        this.levelNames[i] = levelOverheadName;
                }

                this.readFile = readFile;

                } catch(FileNotFoundException e) {
                        System.out.println("File not found.");
                } catch(IOException e) {
                        System.out.println("File IO error.");
                }

        }


        public boolean[] getFlagValues(int level) {
          try{
            boolean[] flagValues = new boolean[7];
            String[] flagNames = {
                "Single-player",
                "Co-operative",
                "Every Man For Himself",
                "Kill the Man with the Ball",
                "King of the Hill",
                "Defense",
                "Rugby"};
            //Game flags:
            /*
                             sp = 0x1;
                             coop = 0x2;
                             emfh = 0x4;
                             ktmwtb = 0x8;
                             koth = 0x10;
                             defense = 0x20;
                             rugby = 0x40;
             */
            int flags = this.flags[level];
            for (int j = 0; j < 7; j++) {
              flagValues[j] = ( (flags & (1 << j)) != 0);
            }
            System.out.println(this.levelNames[level]);
            for (int j = 0; j < flagValues.length; j++) {
              if (flagValues[j]) {
                System.out.println("\t" + flagNames[j]);
              }
            }
            return flagValues;
          }catch(Exception e) {}
          return flagValues;
        }

        public void setFlagValues(int level, boolean[] flagValues) {

                String[] flagNames = {	"Single-player",
                                        "Co-operative",
                                        "Every Man For Himself",
                                        "Kill the Man with the Ball",
                                        "King of the Hill",
                                        "Defense",
                                        "Rugby"};
                int[] flagTypes = {
                        0x1, //single-player
                        0x2, //co-op
                        0x4, //EMFH
                        0x8, //KTMWTB
                        0x10, //KOTH
                        0x20, //Defense
                        0x40, //Rugby
                };
                //int flags = this.flags[level];
                int flags = 0;
                for (int j=0; j < 7; j++) {
                        if (flagValues[j]) {
                                flags =	flags | flagTypes[j];
                        }
                }
                this.flags[level] = flags;
        }

        public void writeFlagValues(String outFileString) {
                try {
                        File outFile = new File(outFileString);
                        RandomAccessFile writeFile = new RandomAccessFile(outFile, "rw");
                        for(int i=0; i < this.levels; i++) {
                                //read trailer for each level:
                                //set seek position to level num * 84 (length of each trailer)
                                long seekPos = mapLength + (i * 84);
                                writeFile.seek(seekPos);
                                short flags = (short) this.flags[i];
                                int levelOffset = writeFile.readInt();
                                int levelLength = writeFile.readInt();
                                int skips = writeFile.skipBytes(8);
                                writeFile.writeShort(flags);
                        }
                        writeFile.close();
                } catch(FileNotFoundException e) {
                        System.out.println("File not found.");
                } catch(IOException e) {
                        System.out.println("File IO error.");
                        System.out.println(e);
                }
        }

        public void closeFile() {
                try {
                        this.readFile.close();
                } catch(FileNotFoundException e) {
                        System.out.println("File not found.");
                } catch(IOException e) {
                        System.out.println("File IO error.");
                }
        }

        public short getLevels() {
                return this.levels;
        }

        public String getLevelName(int level) {
                return this.levelNames[level];
        }

}
